import { get, set, del, update, createStore, values } from 'idb-keyval'
import { Span, Button, List, ListItem, TextArea, createElement, Input } from './weigets.js'

export default class Chat {
    constructor({ name, EventListeners = {}, onsend, onexit }) {
        this.event = { onsend, onexit }
        this.store = createStore(`db-chat-${name}`, `store-chat-${name}`)
        this.ul = List({ classList: ['chat-list'] })
        this.EventListeners = EventListeners
        this.element = createElement({
            style: {
                position: 'fixed', bottom: 0, left: '50%',
                maxWidth: '64rem', width: '100%',
                transform: 'translate(-50%, 0)',
                display: 'none',
                transition: 'all .15s',
            },
            children: [
                this.ul,
                createElement({
                    style: {
                        display: 'flex',
                        margin: '1rem 2rem',
                    },
                    children: [
                        Input({
                            type: 'text',
                            // 随机显示一句话
                            placeholder: [
                                '输入聊天内容',
                                'ctrl+左键: 左边显示',
                                'ctrl+右键: 右边显示',
                                'ctrl+下键: 居中显示',
                                'ctrl+L: 清屏',
                            ].sort(() => Math.random() - 0.5)[0],
                            style: {
                                flex: 1,
                                height: '3rem',
                                padding: '0 1rem',
                                boxSizing: 'border-box',
                                boxShadow: '0 0 1rem #eee',
                                border: 'none',
                                outline: 'none',
                            },
                            onkeydown: event => {
                                event.stopPropagation()
                                const text = event.target.value.trim()
                                if (text && event.key === 'Enter') {
                                    this.发送消息(text)
                                    event.target.value = ''
                                }
                                if (!text && event.key === 'Enter') {
                                    event.target.parentNode.parentNode.style.display = 'none'
                                }
                                if (event.ctrlKey && event.key === 'ArrowLeft') {
                                    event.target.parentNode.parentNode.style.left = '0'
                                    event.target.parentNode.parentNode.style.right = '50%'
                                }
                                if (event.ctrlKey && event.key === 'ArrowRight') {
                                    event.target.parentNode.parentNode.style.left = '50%'
                                    event.target.parentNode.parentNode.style.right = '0'
                                }
                                if (event.ctrlKey && event.key === 'ArrowDown') {
                                    event.target.parentNode.parentNode.style.left = '50%'
                                    event.target.parentNode.parentNode.style.right = '50%'
                                }
                                if (event.ctrlKey && event.key === 'l') {
                                    this.ul.innerHTML = ''
                                }
                            }
                        }),
                        Button({
                            textContent: '发送(Enter)',
                            onclick: event => {
                                const text = event.target.previousSibling.value.trim()
                                if (text) {
                                    this.发送消息(text)
                                    event.target.previousSibling.value = ''
                                }
                            },
                            style: {
                                padding: '.5rem 1rem',
                                boxSizing: 'border-box',
                                boxShadow: '0 0 1rem #eee',
                                borderRadius: '1rem',
                            }
                        }),
                    ]
                })
            ]
        })
        document.body.appendChild(this.element)
        document.head.appendChild(createElement({
            innerText: `
            ul.chat-list {
                max-height: 70vh;
                overflow-y: auto;
            }
            ul.chat-list > li > span {
                cursor: pointer;
            }
            ul.chat-list > li.play > span {
                color: #02be08;
            }
            ul.chat-list > li.cache::marker {
                color: #02be08;
                font-size: 1em;
                contentx: '⚡';
            }
            ul.chat-list > li.disable {
                color: #888;
            }`
        }, 'style'))
        this.从本地载入消息()
        this.挂载全局快捷键()
    }
    async 挂载全局快捷键() {
        document.addEventListener('keydown', event => {
            if (event.key === 'Enter' || event.key === '/') {
                this.element.style.display = 'block'
                this.element.querySelector('input').focus()
            }
        })
    }
    async 从本地载入消息() {
        const data = await values(this.store)
        data.map(item => ({ timestamp: new Date(item.time).getTime(), ...item })).sort((a, b) => a.timestamp - b.timestamp).forEach(item => {
            this.添加元素(item)
        })
    }
    async 筛选指定范围的消息({ start, end }) {
        const data = await values(this.store)
        return data.map(item => ({ timestamp: new Date(item.time).getTime(), ...item })).filter(item => {
            const timestamp = new Date(item.time).getTime()
            return timestamp >= start && timestamp <= end
        })
    }
    async 合并消息列表(list) {
        // 检查本地不存在的id,存储消息
        const data = await values(this.store)
        const ids = data.map(item => item.id)
        list.filter(item => !ids.includes(item.id)).forEach(item => {
            this.添加元素(item)
            this.存储消息(item)
        })
    }
    // 收到应答(对方确认消息已被接收)
    answer(data) {
        const { name, text, time, type } = data
        const item = this.add({ name, text, time, type })
        item.classList.add('disable')
    }
    // 添加一条消息
    add({ name, text, time, type }) {
        const item = ListItem({
            classList: [type],
            children: [
                Span({ textContent: `${name} ${time} ${text}` })
            ]
        })
        this.ul.appendChild(item)
        this.ul.scrollTop = this.ul.scrollHeight
        return item
    }
    send(text) {
        if (this.event.onsend) {
            this.event.onsend(text)
        }
    }
    添加元素(data) {
        this.ul.appendChild(ListItem({
            style: {
                display: 'flex',
                justifyContent: 'space-between',
                alignItems: 'center',
                margin: '1rem',
                padding: '.5rem 1rem',
                boxSizing: 'border-box',
                boxShadow: '0 0 1rem #eee',
                maxWidth: '24rem',
                borderRadius: '1rem',
            },
            children: [
                Span({ textContent: `${data.name} ${data.time} ${data.text}` }),
                Button({
                    style: {
                        boxSizing: 'border-box',
                        boxShadow: '0 0 1rem #eee',
                        borderRadius: '1rem',
                        fontSize: '12px',
                        color: '#555'
                    },
                    textContent: '删除',
                    onclick: event => {
                        event.target.parentNode.remove()
                        del(data.id, this.store)
                    }
                })
            ]
        }))
    }
    存储消息(data) {
        set(data.id, data, this.store)
    }
    发送消息(text) {
        const name = '我'
        const id = window.crypto.randomUUID()
        const time = new Date().toLocaleString()
        const type = 'text'
        const data = { id, name, text, time, type }
        this.添加元素(data)
        this.存储消息(data)
        this.send(data)
    }
    收到消息(data) {
        console.log('收到消息', data)
        this.添加元素(data)
        this.存储消息(data)
    }
}